;*************************************************
; FILE: PICKREC.SC
;
; Script to add record picking PROCs to a library
;
;        Copyright 1994 S/Wizardry, Ltd.
;              All Rights Reserved.

;*************************************************
; PROC: PickRec.AddArray
PROC PickRec.AddArray(PickRec.Info, PickRec.Array)
PRIVATE TagValue
   IF NOT ISASSIGNED(PickRec.Magic) THEN
     PickRec.Magic = "xyzzy"
   ENDIF
   PickRec.Info["Magic"] = PickRec.Magic

   IF NOT PickRec.ValidInfo(PickRec.Info) THEN
     RETURN False
   ENDIF
   MOVETO PickRec.Info["Table"]
   IF ISASSIGNED(PickRec.Info["Index"])       AND
      ISASSIGNED(PickRec.Info["Index Field"]) THEN
     MOVETO FIELD PickRec.Info["Index Field"]
     ORDERTABLE SELECT PickRec.Info["Index"]
     HOME
   ENDIF
   PickRec.Info["RecNo"] = RECNO()
   PickRec.ResyncArray(PickRec.Info, PickRec.Array)
   PickRec.ClearSearch()

   RETURN True
ENDPROC ; PickRec.AddArray

IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.AddArray
  RELEASE PROCS PickRec.AddArray
ENDIF

;*************************************************
; PROC: PickRec.DialogProc
PROC PickRec.DialogProc(TriggerType, TagValue,
                        EventValue, ElementValue,
                        PickRec.Info,
                        PickRec.Array)
   PRIVATE ProcIndex
   ProcIndex = IIF(TriggerType = "EVENT",
                   EventValue["TYPE"],
                   TriggerType)

   IF NOT PickRec.ValidInfo(PickRec.Info) THEN
      QUIT "Control array bad"
   ENDIF

   IF TagValue <> PickRec.Info["Tag"] THEN
     RETURN
   ENDIF

   PickRec.Info["RetVal"] = True
   IF NOT ISASSIGNED(PickRec.Procs) THEN
     PickRec.InitProcs()
   ENDIF
   IF ISASSIGNED(PickRec.Procs[ProcIndex]) THEN
     EXECPROC(PickRec.Procs[ProcIndex])
   ENDIF

   RETURN PickRec.Info["RetVal"]
ENDPROC ; PickRec.DialogProc

IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.DialogProc
  RELEASE PROCS PickRec.DialogProc
ENDIF

;*************************************************
; PROC: PickRec.ResyncArray
PROC PickRec.ResyncArray(PickRec.Info,
                         PickRec.Array)
PRIVATE I, HoldRec

   MOVETO PickRec.Info["Table"]
   FOR I FROM 1 TO PickRec.Info["Elements"]
     PickRec.Array[I] = ""
   ENDFOR

   HoldRec = RecNo()
   FOR I FROM 1 TO PickRec.Info["Elements"]
     EXECPROC PickRec.Info["Format Proc"]
     PickRec.Array[I] = PickRec.Format
     IF ATLAST() THEN
       QUITLOOP
     ENDIF
     DOWN
   ENDFOR
   MOVETO RECORD HoldRec

   PickRec.Info["Picked"] = 1

ENDPROC ; PickRec.ResyncArray
;
IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.ResyncArray
  RELEASE PROCS PickRec.ResyncArray
ENDIF

;*************************************************
; PROC: PickRec.RefreshArray
PROC PickRec.RefreshArray(PickRec.Info,
                          PickRec.Array)

  PickRec.ResyncArray(PickRec.Info, PickRec.Array)
  PickRec.HandleUpdate()
  REFRESHCONTROL PickRec.Info["Tag"]

ENDPROC ; PickRec.RefreshArray

IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.RefreshArray
  RELEASE PROCS PickRec.RefreshArray
ENDIF

;*************************************************
; PROC: PickRec.ResyncRecord
PROC PickRec.ResyncRecord(PickRec.Info,
                          PickRec.Array)
   EXECPROC PickRec.Info["Format Proc"]
   PickRec.Array[PickRec.Info["Picked"]] =
                                    PickRec.Format

ENDPROC ; PickRec.ResyncRecord

IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.ResyncRecord
  RELEASE PROCS PickRec.ResyncRecord
ENDIF

;*************************************************
; PROC: PickRec.RefreshRecord
PROC PickRec.RefreshRecord(PickRec.Info,
                           PickRec.Array)
   PickRec.ResyncRecord(PickRec.Info,
                        PickRec.Array)
   PickRec.HandleUpdate()
   REFRESHCONTROL PickRec.Info["Tag"]

ENDPROC ; PickRec.RefreshRecord
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.RefreshRecord
   RELEASE PROCS PickRec.RefreshRecord
ENDIF

;*************************************************
; PROC: PickRec.InitProcs
PROC PickRec.InitProcs()
   DYNARRAY PickRec.Procs[]
   ; Triggers
   PickRec.Procs["OPEN"] = "PickRec.HandleOpen"
   PickRec.Procs["CLOSE"] = "PickRec.HandleClose"
   PickRec.Procs["UPDATE"] = "PickRec.HandleUpdate"
   PickRec.Procs["ARRIVE"] = "PickRec.HandleArrive"
   PickRec.Procs["DEPART"] = "PickRec.HandleDepart"
   PickRec.Procs["SELECT"] = "PickRec.HandleSelect"
   PickRec.Procs["ACCEPT"] = "PickRec.HandleAccept"
   PickRec.Procs["CANCEL"] = "PickRec.HandleCancel"
   ; Events
   PickRec.Procs["KEY"] = "PickRec.HandleKey"
   PickRec.Procs["MOUSE"] = "PickRec.HandleMouse"
;   PickRec.Procs["MESSAGE"] =
;                         "PickRec.HandleMessage"
;   PickRec.Procs["IDLE"] = "PickRec.HandleIdle"

ENDPROC ; PickRec.InitProcs
;
IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.InitProcs
  RELEASE PROCS PickRec.InitProcs
ENDIF

;*************************************************
; PROC: PickRec.HandleOpen
PROC PickRec.HandleOpen()
  PickRec.HandleUpdate()
ENDPROC ; PickRec.HandleOpen

IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.HandleOpen
  RELEASE PROCS PickRec.HandleOpen
ENDIF

;*************************************************
; PROC: PickRec.HandleClose
PROC PickRec.HandleClose()
ENDPROC ; PickRec.HandleClose

IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.HandleClose
  RELEASE PROCS PickRec.HandleClose
ENDIF

;*************************************************
; PROC: PickRec.HandleUpdate
PROC PickRec.HandleUpdate()
   PickRec.Info["RecNo"] = RecNo()
   COPYTODYNARRAY PickRec.Values
   IF ISASSIGNED(PickRec.Info["Update Proc"]) THEN
     EXECPROC PickRec.Info["Update Proc"]
   ENDIF
ENDPROC ; PickRec.HandleUpdate

IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.HandleUpdate
  RELEASE PROCS PickRec.HandleUpdate
ENDIF

;*************************************************
; PROC: PickRec.HandleArrive
PROC PickRec.HandleArrive()
   MOVETO PickRec.Info["Table"]

   ; Reset the index order
   IF ISASSIGNED(PickRec.Info["Index"]) AND
      ISASSIGNED(PickRec.Info["Index Field"]) THEN
     MOVETO FIELD PickRec.Info["Index Field"]
     ORDERTABLE SELECT PickRec.Info["Index"]
   ENDIF

   ; Move to correct record
   MOVETO RECORD PickRec.Info["RecNo"]
ENDPROC ; PickRec.HandleArrive
;
IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.HandleArrive
  RELEASE PROCS PickRec.HandleArrive
ENDIF

;*************************************************
; PROC: PickRec.HandleDepart
PROC PickRec.HandleDepart()
ENDPROC ; PickRec.HandleDepart
;
IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.HandleDepart
  RELEASE PROCS PickRec.HandleDepart
ENDIF

;*************************************************
; PROC: PickRec.HandleSelect
PROC PickRec.HandleSelect()
  IF PickRec.Array[PickRec.Info["Picked"]]<>"" AND
     ISASSIGNED(PickRec.Info["Select Proc"])  THEN
    EXECPROC PickRec.Info["Select Proc"]
  ENDIF
ENDPROC   ; PickRec.HandleSelect
;
IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.HandleSelect
  RELEASE PROCS PickRec.HandleSelect
ENDIF

;*************************************************
; PROC: PickRec.HandleAccept
PROC PickRec.HandleAccept()
ENDPROC ; PickRec.HandleAccept
;
IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.HandleAccept
  RELEASE PROCS PickRec.HandleAccept
ENDIF

;*************************************************
; PROC: PickRec.HandleCancel
PROC PickRec.HandleCancel()
ENDPROC ; PickRec.HandleCancel
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.HandleCancel
   RELEASE PROCS PickRec.HandleCancel
ENDIF

;*************************************************
; PROC: PickRec.HandleKey
PROC PickRec.HandleKey()
   PRIVATE ProcIndex

   ProcIndex = EventValue["KeyCode"]

   ; Initialize key event handler dispatch table,
   ; if necessary
   IF NOT ISASSIGNED(PickRec.KeyProcs) THEN
     PickRec.InitKeyProcs()
   ENDIF
   ; Dispatch key event handler
   IF ISASSIGNED(PickRec.KeyProcs[ProcIndex]) THEN
     EXECPROC(PickRec.KeyProcs[ProcIndex])
   ENDIF

ENDPROC ; PickRec.HandleKey
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.HandleKey
   RELEASE PROCS PickRec.HandleKey
ENDIF

;*************************************************
; PROC: PickRec.InitKeyProcs
PROC PickRec.InitKeyProcs()
   DYNARRAY PickRec.KeyProcs[]

   ; Navigation keys
   PickRec.KeyProcs[ASC("Up")] =
                          "PickRec.HandleUp"
   PickRec.KeyProcs[ASC("Down")] =
                          "PickRec.HandleDown"
   PickRec.KeyProcs[ASC("PgUp")] =
                          "PickRec.HandlePgUp"
   PickRec.KeyProcs[ASC("PgDn")] =
                          "PickRec.HandlePgDn"
   PickRec.KeyProcs[ASC("CtrlPgUp")] =
                          "PickRec.HandleCtrlPgUp"
   PickRec.KeyProcs[ASC("CtrlPgDn")] =
                          "PickRec.HandleCtrlPgDn"
   PickRec.KeyProcs[ASC("Home")] =
                          "PickRec.HandleHome"
   PickRec.KeyProcs[ASC("End")] =
                          "PickRec.HandleEnd"
   ; Keys which affect search value.
   PickRec.KeyProcs[ASC("Backspace")] =
                          "PickRec.HandleBksp"
   ; Printable characters (except 32, blank,
   ; because blank is used for select)
   FOR C FROM 33 TO 126
      PickRec.KeyProcs[C] =
                         "PickRec.HandlePrintable"
   ENDFOR
   ; Commonly used list manipulation keys
   PickRec.KeyProcs[ASC("Ins")] =
                        "PickRec.HandleIns"
   PickRec.KeyProcs[ASC("Del")] =
                        "PickRec.HandleDel"
   PickRec.KeyProcs[ASC("Enter")] =
                        "PickRec.HandleEnter"

ENDPROC ; PickRec.InitKeyProcs
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.InitKeyProcs
   RELEASE PROCS PickRec.InitKeyProcs
ENDIF

;*************************************************
; PROC: PickRec.HandleUp
PROC PickRec.HandleUp()
   ; If at top of table, return
   IF ATFIRST() THEN
     PickRec.Info["RetVal"] = False
     RETURN
   ENDIF
   PickRec.ClearSearch()

   ; Move up one record in table
   UP
   ; If still in current list, update from new
   ; record and return to let Paradox move the
   ; cursor up in the PICKARRAY otherwise, shift
   ; the values down, refresh and update from
   ; the new record and keep Paradox from playing
   ; with the PICKARRAY
   IF PickRec.Info["Picked"] > 1 THEN
      PickRec.HandleUpdate()
   ELSE
      FOR I FROM PickRec.Info["Elements"]
            TO 2 STEP -1
         PickRec.Array[I] = PickRec.Array[I-1]
      ENDFOR
      PickRec.RefreshRecord(PickRec.Info,
                            PickRec.Array)
      PickRec.Info["RetVal"] = False
   ENDIF
ENDPROC ; PickRec.HandleUp
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.HandleUp
   RELEASE PROCS PickRec.HandleUp
ENDIF

;*************************************************
; PROC: PickRec.HandleDown
PROC PickRec.HandleDown()
   ; If at bottom of table, return
   IF ATLAST() THEN
      PickRec.Info["RetVal"] = False
      RETURN
   ENDIF
   PickRec.ClearSearch()
   ; Move down one record
   DOWN

   ; If still in current list, update from new
   ; record and return to let Paradox move the
   ; cursor down in the PICKARRAY otherwise, shift
   ; values up, refresh and update from new
   ; record and keep Paradox from messing with the
   ; PICKARRAY
   IF PickRec.Info["Picked"] <
                    PickRec.Info["Elements"] THEN
     PickRec.HandleUpdate()
   ELSE
     FOR I FROM 1 TO PickRec.Info["Elements"] - 1
       PickRec.Array[I] = PickRec.Array[I+1]
     ENDFOR
     PickRec.RefreshRecord(PickRec.Info,
                           PickRec.Array)
     PickRec.Info["RetVal"] = False
   ENDIF
ENDPROC ; PickRec.HandleDown
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.HandleDown
   RELEASE PROCS PickRec.HandleDown
ENDIF

;*************************************************
;  PickRec.HandlePgUp
PROC PickRec.HandlePgUp()
   ; If at top of table, return
   IF ATFIRST() THEN
     PickRec.Info["RetVal"] = False
     RETURN
   ENDIF

   PickRec.ClearSearch()
   ; Move to top of current page
   FOR I FROM 1 TO PickRec.Info["Picked"] - 1
     UP
   ENDFOR

   ; Move up one page
   FOR I FROM 1 TO PickRec.Info["Elements"]
     IF ATFIRST() THEN
       QUITLOOP
     ENDIF
     UP
   ENDFOR
   ; Update array and dialog
   PickRec.RefreshArray(PickRec.Info,
                        PickRec.Array)
ENDPROC ; PickRec.HandlePgUp
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.HandlePgUp
   RELEASE PROCS PickRec.HandlePgUp
ENDIF

;*************************************************
; PROC: PickRec.HandlePgDn
PROC PickRec.HandlePgDn()
   PickRec.Info["RetVal"] = False
   ; If at bottom of table, return
   IF ATLAST() THEN
     RETURN
   ENDIF
   PickRec.ClearSearch()
   ; Move to last line on page
   FOR I FROM PickRec.Info["Picked"]
         TO PickRec.Info["Elements"] - 1
     IF ATLAST() THEN
       QUITLOOP
     ENDIF
     PickRec.Info["Picked"] =
                        PickRec.Info["Picked"] + 1
     DOWN
   ENDFOR
   ; If last line on page is last record, update
   ; dialog and return otherwise, move to first
   ; record of new page and update array and
   ; dialog
   IF ATLAST() THEN
      PickRec.HandleUpdate()
      REFRESHCONTROL PickRec.Info["Tag"]
   ELSE
      DOWN
      PickRec.RefreshArray(PickRec.Info,
                           PickRec.Array)
   ENDIF
ENDPROC ; PickRec.HandlePgDn
;
IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.HandlePgDn
  RELEASE PROCS PickRec.HandlePgDn
ENDIF

;*************************************************
; PROC: PickRec.HandleCtrlPgUp
PROC PickRec.HandleCtrlPgUp()
  PickRec.ClearSearch()
  HOME
  PickRec.RefreshArray(PickRec.Info,
                       PickRec.Array)
  PickRec.Info["RetVal"] = False
ENDPROC ; PickRec.HandleCtrlPgUp
;
IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.HandleCtrlPgUp
  RELEASE PROCS PickRec.HandleCtrlPgUp
ENDIF

;*************************************************
; PROC: PickRec.HandleCtrlPgDn
PROC PickRec.HandleCtrlPgDn()
PRIVATE HoldRec, I

  PickRec.ClearSearch()
  END
  HoldRec = RecNo()

  ; If moved off page, resync array
  IF RecNo() - PickRec.Info["RecNo"] >=
                    PickRec.Info["Elements"] THEN
     FOR I FROM 1 TO PickRec.Info["Elements"] - 1
       IF ATFIRST() THEN
         QUITLOOP
       ENDIF
       UP
     ENDFOR
     PickRec.ResyncArray(PickRec.Info,
                          PickRec.Array)
  ENDIF

  ; There may be less than a full page,
  ; so don't move to last element, move to
  ; last non-blank element
  PickRec.Info["Picked"] =
                          PickRec.Info["Elements"]
  WHILE PickRec.Info["Picked"] > 1 AND
        PickRec.Array[PickRec.Info["Picked"]] = ""
    PickRec.Info["Picked"] =
                        PickRec.Info["Picked"] - 1
  ENDWHILE
  MOVETO RECORD HoldRec
  PickRec.HandleUpdate()
  REFRESHCONTROL PickRec.Info["Tag"]
  PickRec.Info["RetVal"] = False
ENDPROC ; PickRec.HandleCtrlPgDn
;
IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.HandleCtrlPgDn
  RELEASE PROCS PickRec.HandleCtrlPgDn
ENDIF

;*************************************************
; PROC: PickRec.HandleHome
PROC PickRec.HandleHome()
   ; If at top of table, return
   IF ATFIRST() THEN
     PickRec.Info["RetVal"] = False
     RETURN
   ENDIF

   PickRec.ClearSearch()
   ; Move to top of list
   FOR I FROM 1 TO PickRec.Info["Picked"] - 1
     UP
   ENDFOR
   PickRec.HandleUpdate()

   ; Let Paradox move the cursor to the
   ; first element

ENDPROC ; PickRec.HandleHome
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.HandleHome
   RELEASE PROCS PickRec.HandleHome
ENDIF

;*************************************************
; PROC: PickRec.HandleEnd
PROC PickRec.HandleEnd()
   PRIVATE I

   PickRec.Info["RetVal"] = False

   ; If at bottom of table, return
   IF ATLAST() THEN
      RETURN
   ENDIF

   PickRec.ClearSearch()
   ; Move to bottom of list
   FOR I FROM PickRec.Info["Picked"]
         TO PickRec.Info["Elements"] - 1
     IF ATLAST() THEN
       QUITLOOP
     ENDIF
     DOWN
   ENDFOR

   PickRec.HandleUpdate()

   ; There may be less than a full page so don't
   ; move to the last element, move to the last
   ; non-blank element
   PickRec.Info["Picked"] =
                          PickRec.Info["Elements"]
   WHILE PickRec.Info["Picked"] > 1 AND
         PickRec.Array[PickRec.Info["Picked"]] = ""
     PickRec.Info["Picked"] =
                       PickRec.Info["Picked"] - 1
   ENDWHILE
   REFRESHCONTROL PickRec.Info["Tag"]
ENDPROC ; PickRec.HandleEnd
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.HandleEnd
   RELEASE PROCS PickRec.HandleEnd
ENDIF

;*************************************************
; PROC: PickRec.HandlePrintable
PROC PickRec.HandlePrintable()
PRIVATE  L, Prefix, PickRec.SearchValue
  IF NOT ISASSIGNED(PickRec.Info["Search Value"])
    THEN PickRec.Info["Search Value"] = ""
  ENDIF

  PickRec.SearchValue =
         PickRec.Info["Search Value"] +
         FORMAT("CU",CHR(EventValue["KEYCODE"]))

  IF PickRec.Search() THEN
    PickRec.Info["Search Value"] =
                              PickRec.SearchValue
    REPAINTDIALOG
  ENDIF
ENDPROC ; PickRec.HandlePrintable
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.HandlePrintable
   RELEASE PROCS PickRec.HandlePrintable
ENDIF

;*************************************************
; PROC: PickRec.HandleBksp
PROC PickRec.HandleBksp()
   PRIVATE L, PickRec.SearchValue
   IF NOT ISASSIGNED(PickRec.Info["Search Value"])
     THEN PickRec.Info["Search Value"] = ""
   ENDIF

   PickRec.SearchValue =
                      PickRec.Info["Search Value"]
   L = LEN(PickRec.SearchValue)
   IF L > 0 THEN
     PickRec.SearchValue =
               SUBSTR(PickRec.SearchValue, 1, L-1)
   ENDIF

   IF PickRec.Search() THEN
     PickRec.Info["Search Value"] =
                              PickRec.SearchValue
     REPAINTDIALOG
   ENDIF
ENDPROC ; PickRec.HandleBksp
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.HandleBksp
   RELEASE PROCS PickRec.HandleBksp
ENDIF

;*************************************************
; PROC: PickRec.ClearSearch
PROC PickRec.ClearSearch()
   PickRec.Info["Search Value"] = ""
   REPAINTDIALOG
ENDPROC ; PickRec.ClearSearch
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.ClearSearch
   RELEASE PROCS PickRec.ClearSearch
ENDIF

;*************************************************
; PROC: PickRec.Search
PROC PickRec.Search()
   PRIVATE FirstEl, LastElement

   ; Paradox should not do any searching
   PickRec.Info["RetVal"] = False

   ; If the new search value is outside of the
   ; array, call the user-provided search function
   FirstElement =
         FORMAT("CU", PickRec.Array[1])
   LastElement =
      FORMAT("CU",
         PickRec.Array[PickRec.Info["Elements"]])
   ; WUZ - This may test against a blank element
   ; if there is less than one page.
   IF PickRec.SearchValue > LastElement OR
      PickRec.SearchValue < FirstElement THEN
     IF ISASSIGNED(PickRec.Info["Search Proc"])
      THEN
         EXECPROC PickRec.Info["Search Proc"]
         IF PickRec.RetVal THEN
            PickRec.RefreshArray(PickRec.Info,
                                 PickRec.Array)
            RETURN True
         ENDIF
     ENDIF
     RETURN False
   ENDIF

   ; Search value should be in the array
   ; if it exists.

   L = LEN(PickRec.SearchValue)
   FOR I FROM 1 TO PickRec.Info["Picked"] - 1
      UP
   ENDFOR
   FOR I FROM 1 TO PickRec.Info["Elements"]
      Prefix = SUBSTR(PickRec.Array[I], 1, L)
      Prefix = FORMAT("CU", Prefix)
      IF Prefix = PickRec.SearchValue THEN
         PickRec.Info["Picked"] = I
         PickRec.HandleUpdate()
         REFRESHCONTROL TagValue
         RETURN True
      ENDIF
      DOWN
   ENDFOR
   MOVETO RECORD PickRec.Info["RecNo"]
   RETURN False
ENDPROC ; PickRec.Search
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.Search
   RELEASE PROCS PickRec.Search
ENDIF

;*************************************************
; PROC: PickRec.ValidInfo
PROC PickRec.ValidInfo(PickRec.Info)
   ; PickRec.Info must be a dynarray
   IF TYPE(PickRec.Info) <> "DY" THEN
      RETURN False
   ENDIF

   ; Moderately lame sanity check
   IF NOT ISASSIGNED(PickRec.Info["Magic"])
   OR PickRec.Info["Magic"] <> PickRec.Magic THEN
      RETURN False
   ENDIF

   ; The tag must be specified
   IF NOT ISASSIGNED(PickRec.Info["Tag"]) THEN
      RETURN False
   ENDIF

   ; The table must be specified
   IF NOT ISASSIGNED(PickRec.Info["Table"])
   OR NOT ISTABLE(PickRec.Info["Table"]) THEN
      RETURN False
   ENDIF

   RETURN True
ENDPROC ; PickRec.ValidInfo
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.ValidInfo
   RELEASE PROCS PickRec.ValidInfo
ENDIF

;*************************************************
; PROC: PickRec.HandleIns
PROC PickRec.HandleIns()
   IF ISASSIGNED(PickRec.Info["Ins Proc"]) THEN
      EXECPROC PickRec.Info["Ins Proc"]
   ENDIF
ENDPROC ; PickRec.HandleIns
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.HandleIns
   RELEASE PROCS PickRec.HandleIns
ENDIF

;*************************************************
; PROC: PickRec.HandleDel
PROC PickRec.HandleDel()
   IF ISASSIGNED(PickRec.Info["Del Proc"]) THEN
      EXECPROC PickRec.Info["Del Proc"]
   ENDIF
ENDPROC ; PickRec.HandleDel
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.HandleDel
   RELEASE PROCS PickRec.HandleDel
ENDIF

;*************************************************
; PROC: PickRec.HandleEnter
PROC PickRec.HandleEnter()
   IF ISASSIGNED(PickRec.Info["Enter Proc"]) THEN
      EXECPROC(PickRec.Info["Enter Proc"])
   ENDIF
ENDPROC ; PickRec.HandleEnter
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.HandleEnter
   RELEASE PROCS PickRec.HandleEnter
ENDIF

;*************************************************
; PROC: PickRec.HandleMouse
PROC PickRec.HandleMouse()
   PickRec.Info["RetVal"] = False
ENDPROC ; PickRec.HandleMouse
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.HandleMouse
   RELEASE PROCS PickRec.HandleMouse
ENDIF

;*************************************************
; PROC: PickRec.HandleMessage
PROC PickRec.HandleMessage()
ENDPROC ; PickRec.HandleMessage
;
IF ISASSIGNED(LibName) THEN
   WRITELIB LibName PickRec.HandleMessage
   RELEASE PROCS PickRec.HandleMessage
ENDIF

;*************************************************
; PROC: PickRec.HandleIdle
PROC PickRec.HandleIdle()
ENDPROC ; PickRec.HandleIdle
;
IF ISASSIGNED(LibName) THEN
  WRITELIB LibName PickRec.HandleIdle
  RELEASE PROCS PickRec.HandleIdle
ENDIF
