PROC TimeIn(What)
ENDPROC
PROC TimeOut(What)
ENDPROC

; Create the PickRec procs
PLAY "PICKREC"

; Each record in list is LName,Fname
PROC T.Format()
  PickRec.Format = [Last Name]+","+[First name]
ENDPROC   ; T.Format

; Search for name in table
PROC T.Search()
PRIVATE P, LName, FName
  
  P = SEARCH(",", PickRec.SearchValue)
  IF P = 0 THEN
    LOCATE INDEXORDER BY "By Name" 
           PATTERN PickRec.SearchValue+".."
  ELSE
    L = LEN(PickRec.SearchValue)
    LName = SUBSTR(PickRec.SearchValue,1,P-1)
    FName = SUBSTR(PickRec.SearchValue,P+1,L-P)
    
    LOCATE INDEXORDER By "By Name" LName
    Found = Retval
    WHILE Retval AND [First Name] < FName 
      LOCATE INDEXORDER By "By Name" NEXT LName      
    ENDWHILE
    Retval = Found 
          AND SUBSTR([First Name],1,LEN(FName)) = FName
  ENDIF
  PickRec.Retval = Retval
ENDPROC   ; T.Search

T.ArrSize = 10

; Load and order table
VIEW "T"
COEDITKEY
MOVETO [Last Name]
ORDERTABLE {By Name}
HOME

; Set up data array and control dynarray for PickRec
ARRAY T1.Array[T.ArrSize]
DYNARRAY T1.Info[]
T1.Info["Table"]       = TABLE()
T1.Info["Tag"]         = "List 1"
T1.Info["Elements"]    = T.ArrSize
T1.Info["Format Proc"] = "T.Format"
T1.Info["Search Proc"] = "T.Search"
T1.Info["Select Proc"] = "T.Select"
T1.Info["Ins Proc"]    = "T.Insert"
T1.Info["Del Proc"]    = "T.Delete"
T1.Info["Update Proc"] = "T.Update"

; Initialize PickRec
PickRec.AddArray(T1.Info,T1.Array)
; Fill array for standard Paradox handling
HOME
ARRAY T2.Array[NRECORDS(TABLE())]
I = 1
SCAN
  T2.Array[I] = [Last Name]+","+[First Name]
  I = I + 1
ENDSCAN
HOME

; PickRec handles events for List 1
PROC T.DialogProc(TriggerType,TagValue,
                  EventValue,ElementValue)
  IF TagValue = T1.Info["Tag"] THEN
    RETURN PickRec.DialogProc(
             TriggerType,TagValue,
             EventValue,ElementValue,
             T1.Info,T1.Array)
  ELSE
    RETURN True
  ENDIF
ENDPROC   ; T.DialogProc

PROC T.Select() 
  IF T.Edit() THEN
    COPYFROMDYNARRAY PickRec.Values
    POSTRECORD
    PickRec.ResyncRecord(T1.Info,T1.Array)
    REFRESHCONTROL T1.Info["Tag"]
  ENDIF
ENDPROC   ; T.Select

PROC T.Insert()
PRIVATE HoldField
  
  ; Give a blank default
  FOREACH Element IN PickRec.Values
    PickRec.Values[Element] = ""
  ENDFOREACH
  
  ; Poor, slow way to generate unique IDs
  HoldField = FIELD()
  MOVETO [ID]
  PickRec.Values["ID"] = IMAGECMAX()+1
  MOVETO FIELD HoldField

  IF T.Edit() THEN
    INS
    COPYFROMDYNARRAY PickRec.Values
    POSTRECORD
    ; Make new record current
    HoldField = FIELD()
    MOVETO [ID]
    LOCATE PickRec.Values["ID"]
    MOVETO FIELD HoldField
    PickRec.RefreshArray(T1.Info,T1.Array)
  ENDIF
ENDPROC   ; T.Insert

PROC T.Edit()
  SHOWDIALOG "Name"
    @ 5,39 HEIGHT 11 WIDTH 38
    
    LABEL @ 1,2 "~L~ast:" FOR "Last Name"
    ACCEPT 
      @ 1,12 WIDTH 23 "A20" 
      TAG "Last Name"
      TO PickRec.Values["Last Name"]
    
    LABEL @ 3,2 "~F~irst:" FOR "First Name"
    ACCEPT 
      @ 3,12 WIDTH 15 "A12" 
      TAG "First Name"
      TO PickRec.Values["First Name"]

    LABEL @ 5,2 "~N~ick:" FOR "NickName"
    ACCEPT 
      @ 5,12 WIDTH 13 
      "A10" 
      TAG "NickName" TO PickRec.Values["NickName"]

    PUSHBUTTON
      @ 7,3 WIDTH 10 
      "~O~K" OK VALUE True 
      TAG "OK BUTTON" TO ButtonVal

    PUSHBUTTON
      @ 7,24 WIDTH 10 
      "~C~ancel" CANCEL VALUE False 
      TAG "CANCEL BUTTON" TO ButtonVal
  ENDDIALOG
  RETURN Retval
ENDPROC   ; T.Edit

PROC T.Delete()
IF ConfirmDelete(T1.Array[T1.Info["Picked"]]) THEN
    DEL
    PickRec.RefreshArray(T1.Info, T1.Array)
  ENDIF
ENDPROC   ; T.Delete

PROC ConfirmDelete(Name) 
  SHOWDIALOG "Confirm Delete"
    @ 4,12 HEIGHT 8 WIDTH 32
    @ 1,1 ?? FORMAT("W29,AC",Name+"?")
      
    PUSHBUTTON @4,2 WIDTH 11
      "~O~K" OK
      DEFAULT VALUE True
      TAG "" TO Retval

    PUSHBUTTON @4,17 WIDTH 12
      "~C~ancel" CANCEL
      VALUE False
      TAG "" TO Retval
  ENDDIALOG
  RETURN Retval
ENDPROC   ; ConfirmDelete

PROC T.Update()
  T.Prompt1 = [First Name] + " " +
              IIF(ISBLANK([Nickname]),"","\"" +
              [Nickname]+"\" ")+
              [Last Name]
  REPAINTDIALOG
ENDPROC   ; T.Update

T.Prompt1 = ""

SHOWDIALOG "PickRec Test"
  PROC "T.DialogProc" ALL
  @ 2,5 HEIGHT 20 WIDTH 70
  
  LABEL @ 1,13 "Pick~R~ec"
  FOR T1.Info["Tag"]
  PICKARRAY
    @ 2,2 HEIGHT T.ArrSize WIDTH 30
    T1.Array
    TAG T1.Info["Tag"] TO T1.Info["Picked"]
  
  ; Show the user what the search value is
  @ 15,2 ?? T1.Info["Search Value"]
    
  LABEL @ 1,43 "~P~aradox"
  FOR "List 2"
  PICKARRAY
    @ 2,36 HEIGHT T.ArrSize WIDTH 30
    T2.Array
    TAG "List 2" TO T2.Picked
  
  @ 13,2 ?? T.Prompt1
  
  PUSHBUTTON
    @ 16,5 WIDTH 15
    "~O~K" OK
    DEFAULT VALUE "OK"
    TAG "OK" TO T.Button
  
  PUSHBUTTON
    @16,35 WIDTH 15
    "~C~ancel" CANCEL
    VALUE "CANCEL"
    TAG "CANCEL" TO T.Button
ENDDIALOG

IF Retval THEN
  MESSAGE PickRec.Values["ID"]," ",
          PickRec.Values["First Name"]," ",
          PickRec.Values["Last name"]
  x = GETCHAR()
ENDIF

Do_It!
CLEARIMAGE

